@@ -64,6 +64,8 @@ https://blockattack.net
 #define PACKAGE "blockattack_roftb"
 #endif
 
+#define MODLIST_TXT "mod_list.txt"
+
 #include "highscore.h"      //Stores highscores
 #include "ReadKeyboard.h"   //Reads text from keyboard
 #include "stats.h"          //Saves general stats
@@ -71,6 +73,7 @@ https://blockattack.net
 
 #include "common.h"
 #include "gamecontroller.h"
+#include <boost/algorithm/string.hpp>
 #include <boost/program_options.hpp>
 #include <fstream>
 #include "levelselect.hpp"
@@ -90,6 +93,15 @@ static void FsSearchParthMainAppend(std::vector<std::string>& paths) {
 	paths.push_back((std::string)PHYSFS_getBaseDir()+"/data");
 }
 
+static void FsSearchPathModAppend(std::vector<std::string>& paths, const std::vector<std::string>& modlist) {
+	for (const std::string& mod : modlist) {
+		std::string filename = std::string(SHAREDIR)+"/mods/"+mod+".data";
+		paths.push_back(filename);
+		filename = std::string(PHYSFS_getBaseDir())+"/mods/"+mod+".data";
+		paths.push_back(filename);
+	}
+}
+
 static void PhysFsSetSearchPath(const std::vector<std::string>& paths, const std::string& savepath) {
 	for (const std::string& path : paths) {
 		PHYSFS_mount(path.c_str(), "/", 0);
@@ -210,7 +222,8 @@ static int InitImages(sago::SagoSpriteHolder& holder) {
 		globalData.typingChunk = holder.GetDataHolder().getSoundHandler("typing");
 		counterChunk = holder.GetDataHolder().getSoundHandler("counter");
 		counterFinalChunk = holder.GetDataHolder().getSoundHandler("counter_final");
-		const int soundVolume = 84;  //0-128
+		Config::getInstance()->setDefault("volume_sound", "24"); //0-128
+		int soundVolume = Config::getInstance()->getInt("volume_sound");
 		Mix_VolumeChunk(boing.get(), soundVolume);
 		Mix_VolumeChunk(applause.get(), soundVolume);
 		Mix_VolumeChunk(photoClick.get(), soundVolume);
@@ -264,6 +277,7 @@ void ResetFullscreen() {
 	SDL_RenderSetLogicalSize(globalData.screen, globalData.xsize, globalData.ysize);
 	dataHolder.invalidateAll(globalData.screen);
 	globalData.spriteHolder.reset(new sago::SagoSpriteHolder( dataHolder ) );
+	globalData.spriteHolder->ReadSprites(globalData.modinfo.getModSpriteFiles());
 	InitImages(*(globalData.spriteHolder.get()) );
 	SDL_ShowCursor(SDL_DISABLE);
 }
@@ -334,24 +348,10 @@ void writeScreenShot() {
 		std::cout << "Saving screenshot" << "\n";
 	}
 	int rightNow = (int)time(nullptr);
-	SDL_Surface* infoSurface = SDL_GetWindowSurface(sdlWindow);
-	if (!infoSurface) {
-		std::cerr << "Could not get infoSurface. No screenshot written. Be aware that the screenshot feature only works with software render\n";
-		return;
-	}
-	std::vector<char> pixels(infoSurface->w * infoSurface->h * infoSurface->format->BytesPerPixel);
-	int errorCode = SDL_RenderReadPixels(globalData.screen, &infoSurface->clip_rect, infoSurface->format->format, static_cast<void*>(pixels.data()), infoSurface->w * infoSurface->format->BytesPerPixel);
-	if (errorCode) {
-		SDL_FreeSurface(infoSurface);
-		std::cerr << "Could not do SDL_RenderReadPixels. Error code: " << errorCode << ". No screenshot written\n";
-		return;
-	}
-	SDL_Surface* sreenshotSurface = SDL_CreateRGBSurfaceFrom(static_cast<void*>(pixels.data()), infoSurface->w, infoSurface->h, infoSurface->format->BitsPerPixel, infoSurface->w * infoSurface->format->BytesPerPixel, infoSurface->format->Rmask, infoSurface->format->Gmask, infoSurface->format->Bmask, infoSurface->format->Amask);
-	SDL_FreeSurface(infoSurface);
-	if (!sreenshotSurface) {
-		std::cerr << "Could not get sreenshotSurface. No screenshot written\n";
-		return;
-	}
+	int w, h;
+	SDL_GetRendererOutputSize(globalData.screen, &w, &h);
+	SDL_Surface *sreenshotSurface = SDL_CreateRGBSurface(0, w, h, 32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000);
+	SDL_RenderReadPixels(globalData.screen, NULL, SDL_PIXELFORMAT_ARGB8888, sreenshotSurface->pixels, sreenshotSurface->pitch);
 	OsCreateFolder(pathToScreenShots());
 	std::string buf = pathToScreenShots() + "/screenshot"+std::to_string(rightNow)+".bmp";
 	SDL_SaveBMP(sreenshotSurface, buf.c_str());
@@ -866,6 +866,7 @@ static void ParseArguments(int argc, char* argv[], globalConfig& conf) {
 	        "Default: \"%s\"", LOCALEDIR).c_str())
 	("homepath", boost::program_options::value<std::string>(), SPrintStringF("Set the home folder where settings are saved. The directory will be created if it does not exist."
 	        " Default: \"%s\"", getPathToSaveFiles().c_str()).c_str())
+	("mod,m", boost::program_options::value<std::vector<std::string> >(), "Loads a mod. Later mods have preference")
 
 	;
 	boost::program_options::variables_map vm;
@@ -957,9 +958,17 @@ static void ParseArguments(int argc, char* argv[], globalConfig& conf) {
 	if (vm.count("play-replay")) {
 		globalData.replayArgument = vm["play-replay"].as<std::string>();
 	}
+	if (vm.count("mod")) {
+		globalData.modList = vm["mod"].as<std::vector<std::string> >();
+	}
 
 }
 
+//Physfs 2.0.z does not have PHYSFS_unmount
+#if (PHYSFS_VER_MAJOR <= 2) && (PHYSFS_VER_MINOR < 1)
+#define PHYSFS_unmount PHYSFS_removeFromSearchPath
+#endif
+
 //Warning: the arguments to main must be "int argc, char* argv[]" NO CONST! or SDL_main will fail to find it
 int main(int argc, char* argv[]) {
 	try {
@@ -979,6 +988,25 @@ int main(int argc, char* argv[]) {
 		ParseArguments(argc, argv, config);
 		OsCreateSaveFolder();
 		PhysFsSetSearchPath(config.search_paths, config.savepath);
+		/*if (globalData.modList.empty() && sago::FileExists(MODLIST_TXT))  {
+			std::string modString = sago::GetFileContent(MODLIST_TXT);
+			boost::split(globalData.modList, modString, boost::is_any_of(","));
+		}*/
+		globalData.modinfo.InitModList(globalData.modList);
+		if (sago::FileExists(MODLIST_TXT)) {
+			globalData.modinfo.ParseModFile(sago::GetFileContent(MODLIST_TXT));
+			globalData.modList = globalData.modinfo.getModList();
+		}
+		std::cout << "Mod list: ";
+		for (const std::string& s : globalData.modList) {
+			std::cout << s << ",";
+		}
+		std::cout <<  "\n";
+		if (globalData.modList.size()>0) {
+			PHYSFS_unmount(config.savepath.c_str());
+			FsSearchPathModAppend(config.search_paths, globalData.modList);
+			PhysFsSetSearchPath(config.search_paths, config.savepath);
+		}
 		//Os create folders must be after the parameters because they can change the home folder
 		PhysFsCreateFolders();
 		bool gameShutdownProperly = true;
@@ -1015,6 +1043,7 @@ int main(int argc, char* argv[]) {
 				globalData.NoSound = true; //Tries to stop all sound from playing/loading
 			}
 		}
+		Config::getInstance()->setDefault("volume_music", "20"); //0-128
 
 
 		if (globalData.verboseLevel) {
@@ -1245,11 +1274,19 @@ int main(int argc, char* argv[]) {
 			std::cout << SPrintStringF("Block Attack - Rise of the Blocks ran for: %i hours %i mins and %i secs", ct.hours, ct.minutes, ct.seconds) << "\n";
 		}
 
+		/*std::string modListString;
+		if (globalData.modList.size()>0) {
+			modListString = globalData.modList.at(0);
+			for (size_t i = 1; i < globalData.modList.size(); ++i) {
+				modListString += std::string(",")+globalData.modList[i];
+			}
+		}*/
 		ct = TimeHandler::addTime("totalTime",ct);
 		if (globalData.verboseLevel) {
 			std::cout << "Total run time is now: " << ct.days << " days " << ct.hours << " hours " << ct.minutes << " mins and " << ct.seconds << " secs" << "\n";
+			//std::cout << "Mods loaded: " << modListString << "\n";
 		}
-
+		//sago::WriteFileContent(MODLIST_TXT, modListString);
 		Stats::getInstance()->save();
 		Config::getInstance()->save();
 	}
@@ -1700,20 +1737,22 @@ int runGame(Gametype gametype, int level) {
 		if ((!globalData.NoSound)&&(!Mix_PlayingMusic())&&(globalData.MusicEnabled)&&(!bNearDeath)) {
 			// then starts playing it.
 			Mix_PlayMusic(bgMusic.get(), -1); //music loop
-			Mix_VolumeMusic((MIX_MAX_VOLUME*3)/10);
+			int musicVolume = Config::getInstance()->getInt("volume_music");
+			Mix_VolumeMusic(musicVolume);
 		}
 
 		if (bNearDeath!=bNearDeathPrev) {
+			int musicVolume = Config::getInstance()->getInt("volume_music");
 			if (bNearDeath) {
 				if (!globalData.NoSound &&(globalData.MusicEnabled)) {
 					Mix_PlayMusic(highbeatMusic.get(), 1);
-					Mix_VolumeMusic((MIX_MAX_VOLUME*5)/10);
+					Mix_VolumeMusic(musicVolume);
 				}
 			}
 			else {
 				if (!globalData.NoSound &&(globalData.MusicEnabled)) {
 					Mix_PlayMusic(bgMusic.get(), -1);
-					Mix_VolumeMusic((MIX_MAX_VOLUME*3)/10);
+					Mix_VolumeMusic(musicVolume);
 				}
 			}
 		}
