@@ -27,22 +27,17 @@ SOFTWARE.
 #include <iostream>
 #include <iconv.h>
 #include <string.h>
-#include <memory>
 
 #if PHYSFS_VER_MAJOR < 3
 #define PHYSFS_readBytes(X,Y,Z) PHYSFS_read(X,Y,1,Z)
 #define PHYSFS_writeBytes(X,Y,Z) PHYSFS_write(X,Y,1,Z)
 #endif
 
-using std::string;
-using std::cerr;
-using std::vector;
-
 namespace sago {
 
 
 std::vector<std::string> GetFileList(const char* dir) {
-	vector<string> ret;
+	std::vector<std::string> ret;
 	char** rc = PHYSFS_enumerateFiles(dir);
 	for (char** i = rc; *i != NULL; i++) {
 		ret.push_back(*i);
@@ -55,11 +50,11 @@ bool FileExists(const char* filename) {
 	return PHYSFS_exists(filename);
 }
 
-std::string GetFileContent(const char* filename) {
-	string ret;
+void ReadBytesFromFile(const char* filename, std::unique_ptr<char[]>& dest, unsigned int& bytes) {
+	bytes = 0;
 	if (!PHYSFS_exists(filename)) {
-		cerr << "GetFileContent - File does not exists: " << filename << "\n";
-		return ret;
+		std::cerr << "ReadBytesFromFile - File does not exists: " << filename << "\n";
+		return;
 	}
 	PHYSFS_file* myfile = PHYSFS_openRead(filename);
 	unsigned int m_size = PHYSFS_fileLength(myfile);
@@ -67,12 +62,25 @@ std::string GetFileContent(const char* filename) {
 	int length_read = PHYSFS_readBytes (myfile, m_data.get(), m_size);
 	if (length_read != (int)m_size) {
 		PHYSFS_close(myfile);
-		cerr << "Error: Curropt data file: " << filename << "\n";
-		return ret;
+		std::cerr << "Error: Curropt data file: " << filename << "\n";
+		return;
 	}
 	PHYSFS_close(myfile);
-	//Now create a std::string 
-	ret = string(m_data.get(), m_data.get()+m_size);
+	std::swap(m_data, dest);
+	bytes = m_size;
+}
+
+std::string GetFileContent(const char* filename) {
+	std::string ret;
+	if (!PHYSFS_exists(filename)) {
+		std::cerr << "GetFileContent - File does not exists: " << filename << "\n";
+		return ret;
+	}
+	unsigned int m_size = 0;
+	std::unique_ptr<char[]> m_data;
+	ReadBytesFromFile(filename, m_data, m_size);
+	//Now create a std::string
+	ret = std::string(m_data.get(), m_data.get()+m_size);
 	return ret;
 }
 
